/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
import * as core from "@fluidframework/server-services-core";
import { AggregationCursor, Collection, FindOneAndUpdateOptions, FindOptions, MongoClient } from "mongodb";
import { MongoErrorRetryAnalyzer } from "./mongoExceptionRetryRules";
/**
 * @internal
 */
export declare class MongoCollection<T> implements core.ICollection<T>, core.IRetryable {
    private readonly collection;
    readonly retryEnabled: boolean;
    private readonly telemetryEnabled;
    private readonly mongoErrorRetryAnalyzer;
    private readonly collectionName;
    private readonly apiCounterIntervalMS;
    private readonly apiFailureRateTerminationThreshold;
    private readonly apiMinimumCountToEnableTermination;
    private readonly consecutiveFailedThresholdForLowerTotalRequests;
    private readonly isGlobalDb;
    private readonly apiCounter;
    private readonly failedApiCounterSuffix;
    private consecutiveFailedCount;
    constructor(collection: Collection<T>, retryEnabled: boolean, telemetryEnabled: boolean, mongoErrorRetryAnalyzer: MongoErrorRetryAnalyzer, collectionName: string, apiCounterIntervalMS: number, apiFailureRateTerminationThreshold: number, apiMinimumCountToEnableTermination: number, consecutiveFailedThresholdForLowerTotalRequests: number, isGlobalDb?: boolean);
    aggregate(pipeline: any, options?: any): Promise<AggregationCursor<T>>;
    find(query: object, sort: any, limit?: number, skip?: number): Promise<T[]>;
    findOne(query: object, options?: FindOptions): Promise<T>;
    findAll(): Promise<T[]>;
    update(filter: object, set: any, addToSet: any, options?: any): Promise<void>;
    updateMany(filter: object, set: any, addToSet: any, options?: any): Promise<void>;
    upsert(filter: object, set: any, addToSet: any, options?: any): Promise<void>;
    distinct(key: any, query: any): Promise<any>;
    deleteOne(filter: any): Promise<any>;
    deleteMany(filter: any): Promise<any>;
    insertOne(value: T): Promise<any>;
    insertMany(values: T[], ordered: boolean): Promise<void>;
    createIndex(index: any, unique: boolean): Promise<void>;
    createTTLIndex(index: any, expireAfterSeconds?: number): Promise<void>;
    findOrCreate(query: any, value: any, options?: {
        returnOriginal: boolean;
        upsert: boolean;
    }): Promise<{
        value: T;
        existing: boolean;
    }>;
    findAndUpdate(query: any, value: any, options?: FindOneAndUpdateOptions): Promise<{
        value: T;
        existing: boolean;
    }>;
    private updateCore;
    private updateManyCore;
    private requestWithRetry;
    private getTelemetryPropertiesFromQuery;
    private sanitizeError;
    private cloneError;
    private terminateBasedOnCounterThreshold;
}
/**
 * @internal
 */
export declare class MongoDb implements core.IDb {
    private readonly client;
    private readonly retryEnabled;
    private readonly telemetryEnabled;
    private readonly mongoErrorRetryAnalyzer;
    private readonly apiCounterIntervalMS;
    private readonly apiFailureRateTerminationThreshold;
    private readonly apiMinimumCountToEnableTermination;
    private readonly consecutiveFailedThresholdForLowerTotalRequests;
    private readonly isGlobalDb;
    constructor(client: MongoClient, retryEnabled: boolean, telemetryEnabled: boolean, mongoErrorRetryAnalyzer: MongoErrorRetryAnalyzer, apiCounterIntervalMS: number, apiFailureRateTerminationThreshold: number, apiMinimumCountToEnableTermination: number, consecutiveFailedThresholdForLowerTotalRequests: number, isGlobalDb?: boolean);
    close(): Promise<void>;
    on(event: core.IDbEvents, listener: (...args: any[]) => void): void;
    collection<T>(name: string, dbName?: string): core.ICollection<T>;
    healthCheck(dbName?: string): Promise<void>;
    dropCollection(name: string, dbName?: string): Promise<boolean>;
}
/**
 * @internal
 */
export type ConnectionNotAvailableMode = "ruleBehavior" | "stop";
interface IMongoDBConfig {
    operationsDbEndpoint: string;
    globalDbEndpoint?: string;
    globalDbEnabled?: boolean;
    connectionPoolMinSize?: number;
    connectionPoolMaxSize?: number;
    directConnection?: boolean;
    facadeLevelRetry?: boolean;
    facadeLevelTelemetry?: boolean;
    facadeLevelRetryRuleOverride?: any;
    connectionNotAvailableMode?: ConnectionNotAvailableMode;
    dbMonitoringEventsList?: string[];
    heartbeatFrequencyMS?: number;
    keepAliveInitialDelay?: number;
    socketTimeoutMS?: number;
    connectionTimeoutMS?: number;
    minHeartbeatFrequencyMS?: number;
    apiCounterIntervalMS?: number;
    apiFailureRateTerminationThreshold?: number;
    apiMinimumCountToEnableTermination?: number;
    serverSelectionTimeoutMS?: number;
    consecutiveFailedThresholdForLowerTotalRequests: number;
}
/**
 * @internal
 */
export declare class MongoDbFactory implements core.IDbFactory {
    private readonly operationsDbEndpoint;
    private readonly globalDbEndpoint?;
    private readonly connectionPoolMinSize?;
    private readonly connectionPoolMaxSize?;
    private readonly directConnection;
    private readonly retryEnabled;
    private readonly telemetryEnabled;
    private readonly connectionNotAvailableMode;
    private readonly retryRuleOverride;
    private readonly dbMonitoringEventsList;
    private readonly heartbeatFrequencyMS;
    private readonly keepAliveInitialDelay;
    private readonly socketTimeoutMS;
    private readonly connectionTimeoutMS;
    private readonly minHeartbeatFrequencyMS;
    private readonly apiCounterIntervalMS;
    private readonly apiFailureRateTerminationThreshold;
    private readonly apiMinimumCountToEnableTermination;
    private readonly serverSelectionTimeoutMS;
    private readonly consecutiveFailedThresholdForLowerTotalRequests;
    constructor(config: IMongoDBConfig);
    connect(global?: boolean): Promise<core.IDb>;
}
export {};
//# sourceMappingURL=mongodb.d.ts.map